@@ -24,117 +24,122 @@ require_once('iCalendar.php');
 class DAVResource
 {
   /**
-  * @var The partial URL of the resource within our namespace, which this resource is being retrieved as
+  * @var string The partial URL of the resource within our namespace, which this resource is being retrieved as
   */
   protected $dav_name;
 
   /**
-  * @var Boolean: does the resource actually exist yet?
+  * @var bool Does the resource actually exist yet?
   */
   protected $exists;
 
   /**
-  * @var The unique etag associated with the current version of the resource
+  * @var string The unique etag associated with the current version of the resource
   */
   protected $unique_tag;
 
   /**
-  * @var The actual resource content, if it exists and is not a collection
+  * @var string The actual resource content, if it exists and is not a collection
   */
   protected $resource;
 
   /**
-  * @var The parent of the resource, which will always be a collection
+  * @var DAVResource The parent of the resource, which will always be a collection
   */
   protected $parent;
 
   /**
-  * @var The types of the resource, possibly multiple
+  * @var array The types of the resource, possibly multiple
   */
   protected $resourcetypes;
 
   /**
-  * @var The type of the content
+  * @var string The type of the content
   */
   protected $contenttype;
 
   /**
-  * @var The canonical name which this resource exists at
+  * @var string The canonical name which this resource exists at
   */
   protected $bound_from;
 
   /**
-  * @var An object which is the collection record for this resource, or for it's container
+  * @var DAVResource An object which is the collection record for this resource, or for it's container
   */
   private $collection;
 
   /**
-  * @var An object which is the principal for this resource, or would be if it existed.
+  * @var DAVPrincipal An object which is the principal for this resource, or would be if it existed.
   */
   private $principal;
 
   /**
-  * @var A bit mask representing the current user's privileges towards this DAVResource
+  * @var integer A bit mask representing the current user's privileges towards this DAVResource
   */
   private $privileges;
 
   /**
-  * @var True if this resource is a collection of any kind
+  * @var bool True if this resource is a collection of any kind
   */
   private $_is_collection;
 
   /**
-  * @var True if this resource is a principal-URL
+  * @var bool True if this resource is a principal-URL
   */
   private $_is_principal;
 
   /**
-  * @var True if this resource is a calendar collection
+  * @var bool True if this resource is a calendar collection
   */
   private $_is_calendar;
 
   /**
-  * @var True if this resource is a binding to another resource
+  * @var bool True if this resource is a binding to another resource
   */
   private $_is_binding;
 
   /**
-  * @var True if this resource is a binding to an external resource
+  * @var bool True if this resource is a binding to an external resource
   */
   private $_is_external;
 
   /**
-  * @var True if this resource is an addressbook collection
+  * @var bool True if this resource is an addressbook collection
   */
   private $_is_addressbook;
 
   /**
-  * @var True if this resource is, or is in, a proxy collection
+  * @var bool True if this resource is, or is in, a proxy collection
   */
-  private $_is_proxy_request;
+  private $_is_proxy_resource;
 
   /**
-  * @var An array of the methods we support on this resource.
+  * @var The type of proxy collection this resource is or is in: read or write
+  */
+  private $proxy_type;
+
+  /**
+  * @var array An array of the methods we support on this resource.
   */
   private $supported_methods;
 
   /**
-  * @var An array of the reports we support on this resource.
+  * @var array An array of the reports we support on this resource.
   */
   private $supported_reports;
 
   /**
-  * @var An array of the dead properties held for this resource
+  * @var array An array of the dead properties held for this resource
   */
   private $dead_properties;
 
   /**
-  * @var An array of the component types we support on this resource.
+  * @var array An array of the component types we support on this resource.
   */
   private $supported_components;
 
   /**
-  * @var An array of DAVTicket objects if any apply to this resource, such as via a bind.
+  * @var array An array of DAVTicket objects if any apply to this resource, such as via a bind.
   */
   private $tickets;
 
@@ -143,8 +148,11 @@ class DAVResource
   * @param mixed $parameters If null, an empty Resourced is created.
   *     If it is an object then it is expected to be a record that was
   *     read elsewhere.
+  * @param object $prefetched_collection If provided, the internal collection
+  * field of the resource is populated with the given data, so it does not need
+  * to be queried again later
   */
-  function __construct( $parameters = null ) {
+  function __construct( $parameters = null, DAVResource $prefetched_collection = null ) {
     $this->exists        = null;
     $this->bound_from    = null;
     $this->dav_name      = null;
@@ -166,7 +174,12 @@ class DAVResource
     $this->_is_binding       = false;
     $this->_is_external      = false;
     $this->_is_addressbook   = false;
-    $this->_is_proxy_request = false;
+    $this->_is_proxy_resource = false;
+
+    if ( isset($prefetched_collection) ) {
+      $this->collection = $prefetched_collection;
+    }
+
     if ( isset($parameters) && is_object($parameters) ) {
       $this->FromRow($parameters);
     }
@@ -259,12 +272,14 @@ class DAVResource
 
       $this->_is_calendar      = ($this->collection->is_calendar == 't');
       $this->_is_addressbook   = ($this->collection->is_addressbook == 't');
-      $this->_is_proxy_request = ($this->collection->type == 'proxy');
+      $this->_is_proxy_resource = ($this->collection->type == 'proxy');
       if ( $this->_is_principal && !isset($this->resourcetypes) ) {
         $this->resourcetypes   = '<DAV::collection/><DAV::principal/>';
       }
-      else if ( $this->_is_proxy_request ) {
+      else if ( $this->_is_proxy_resource ) {
         $this->resourcetypes  = $this->collection->resourcetypes;
+        preg_match( '#^/[^/]+/calendar-proxy-(read|write)/?[^/]*$#', $this->dav_name, $matches );
+        $this->proxy_type = $matches[1];
       }
       if ( isset($this->collection->dav_displayname) ) $this->collection->displayname = $this->collection->dav_displayname;
     }
@@ -274,6 +289,7 @@ class DAVResource
         if ( isset($this->resource->summary) )$this->resource->displayname = $this->resource->summary;
         if ( strtoupper(substr($this->resource->caldav_data,0,15)) == 'BEGIN:VCALENDAR' ) {
           $this->contenttype = 'text/calendar';
+          if ( isset($this->resource->caldav_type) ) $this->contenttype .= "; component=" . strtolower($this->resource->caldav_type);
           if ( !$this->HavePrivilegeTo('read') && $this->HavePrivilegeTo('read-free-busy') ) {
             $vcal = new iCalComponent($this->resource->caldav_data);
             $confidential = $vcal->CloneConfidential();
@@ -284,7 +300,7 @@ class DAVResource
             $this->resource->url = null;
           }
           else {
-            if ( strtoupper($this->resource->class)=='CONFIDENTIAL' && !$this->HavePrivilegeTo('all') && $session->user_no != $this->resource->user_no ) {
+            if ( isset($this->resource->class) && strtoupper($this->resource->class)=='CONFIDENTIAL' && !$this->HavePrivilegeTo('all') && $session->user_no != $this->resource->user_no ) {
               $vcal = new iCalComponent($this->resource->caldav_data);
               $confidential = $vcal->CloneConfidential();
               $this->resource->caldav_data = $confidential->Render();
@@ -399,12 +415,12 @@ EOSQL;
     }
     else if ( preg_match( '#^(/([^/]+)/calendar-proxy-(read|write))/?[^/]*$#', $this->dav_name, $matches ) ) {
       $this->collection->type = 'proxy';
-      $this->_is_proxy_request = true;
+      $this->_is_proxy_resource = true;
       $this->proxy_type = $matches[3];
       $this->collection->dav_name = $this->dav_name;
       $this->collection->dav_displayname = sprintf( '%s proxy %s', $matches[2], $matches[3] );
       $this->collection->exists = true;
-      $this->collection->parent_container = $matches[1] . '/';
+      $this->collection->parent_container = '/' . $matches[2] . '/';
     }
     else if ( preg_match( '#^(/[^/]+)/?$#', $this->dav_name, $matches)
            || preg_match( '#^((/principals/[^/]+/)[^/]+)/?$#', $this->dav_name, $matches) ) {
@@ -426,8 +442,8 @@ EOSQL;
 SELECT collection.*, path_privs(:session_principal::int8, collection.dav_name,:scan_depth::int), p.principal_id,
     p.type_id AS principal_type_id, p.displayname AS principal_displayname, p.default_privileges AS principal_default_privileges,
     timezones.vtimezone, dav_binding.access_ticket_id, dav_binding.parent_container AS bind_parent_container,
-		dav_binding.dav_displayname, owner.dav_name AS bind_owner_url, dav_binding.dav_name AS bound_to,  
-    dav_binding.external_url AS external_url, dav_binding.type AS external_type, dav_binding.bind_id AS bind_id 
+    dav_binding.dav_displayname, owner.dav_name AS bind_owner_url, dav_binding.dav_name AS bound_to,
+    dav_binding.external_url AS external_url, dav_binding.type AS external_type, dav_binding.bind_id AS bind_id
 FROM dav_binding
     LEFT JOIN collection ON (collection.collection_id=bound_source_id)
     LEFT JOIN principal p USING (user_no)
@@ -480,15 +496,15 @@ EOSQL;
         $this->collection->dav_name = preg_replace('{/[^/]*$}', '/', $this->dav_name);
       }
     }
-    
+
   }
-  
+
   /**
   * Find the collection associated with this resource.
   */
   protected function FetchCollection() {
     global $session;
-    
+
     /**
     * RFC4918, 8.3: Identifiers for collections SHOULD end in '/'
     *    - also discussed at more length in 5.2
@@ -527,7 +543,7 @@ EOSQL;
       }
       @dbg_error_log( 'DAVResource', ':FetchCollection: Read cached collection named "%s" of type "%s".', $this->collection->dav_name, $this->collection->type );
     }
-    
+
     if ( isset($this->collection->bound_from) ) {
       $this->_is_binding = true;
       $this->bound_from = str_replace( $this->collection->bound_to, $this->collection->bound_from, $this->dav_name);
@@ -536,7 +552,7 @@ EOSQL;
         $this->tickets[] = new DAVTicket($this->collection->access_ticket_id);
       }
     }
-    
+
     $this->_is_collection = ( $this->_is_principal || $this->collection->dav_name == $this->dav_name || $this->collection->dav_name == $this->dav_name.'/' );
     if ( $this->_is_collection ) {
       $this->dav_name = $this->collection->dav_name;
@@ -597,8 +613,6 @@ EOSQL;
   * Retrieve the actual resource.
   */
   protected function FetchResource() {
-    global $c, $session;
-
     if ( isset($this->exists) ) return;   // True or false, we've got what we can already
     if ( $this->_is_collection ) return;   // We have all we're going to read
 
@@ -639,6 +653,12 @@ EOQRY;
     }
   }
 
+  /**
+   * FIXME: does this function return a string or an array, or either?
+   * It used to be string only, but b4fd9e2e changed successfully parsed
+   * values to array. However values not in angle brackets are passed
+   * through, and those seem to be the majority in my database?!
+   */
   public static function BuildDeadPropertyXML($property_name, $raw_string) {
     if ( !preg_match('{^\s*<.*>\s*$}s', $raw_string) ) return $raw_string;
     $xmlns = null;
@@ -653,11 +673,17 @@ EOQRY;
     xml_parser_set_option ( $xml_parser, XML_OPTION_CASE_FOLDING, 0 );
     $rc = xml_parse_into_struct( $xml_parser, $xml, $xml_tags );
     if ( $rc == false ) {
-      dbg_error_log( 'ERROR', 'XML parsing error: %s at line %d, column %d',
-                  xml_error_string(xml_get_error_code($xml_parser)),
+      $errno = xml_get_error_code($xml_parser);
+      dbg_error_log( 'ERROR', 'XML parsing error: %s (%d) at line %d, column %d',
+                  xml_error_string($errno), $errno,
                   xml_get_current_line_number($xml_parser), xml_get_current_column_number($xml_parser) );
       dbg_error_log( 'ERROR', "Error occurred in:\n%s\n",$xml);
-      return $raw_string;
+      if ($errno >= 200 && $errno < 300 && count($xml_tags) >= 3) {
+          // XML namespace error, but parsing was probably fine: continue and return tags (cf. #9)
+          dbg_error_log( 'ERROR', 'XML namespace error but tags extracted, trying to continue');
+      } else {
+          return $raw_string;
+      }
     }
     xml_parser_free($xml_parser);
     $position = 0;
@@ -741,9 +767,10 @@ EOQRY;
     return $this->parent;
   }
 
-  
+
   /**
   * Fetch the parent to this resource. This is deprecated - use GetParentContainer() instead.
+  * @deprecated
   */
   function FetchParentContainer() {
     deprecated('DAVResource::FetchParentContainer');
@@ -913,6 +940,7 @@ EOQRY;
       'DAV::principal-property-search' => '',
       'DAV::principal-search-property-set' => '',
       'DAV::expand-property' => '',
+      'DAV::principal-match' => '',
       'DAV::sync-collection' => ''
     );
 
@@ -1090,6 +1118,18 @@ EOQRY;
   }
 
 
+  /**
+  * Checks whether this resource is a proxy collection
+  * @param string $type The type of proxy collection, 'read', 'write' or 'any'
+  */
+  function IsProxyCollection( $type = 'any' ) {
+    if ( $this->_is_proxy_resource ) {
+      return ($type == 'any' || $type == $this->proxy_type);
+    }
+    return false;
+  }
+
+
   /**
   * Checks whether this resource is a scheduling inbox/outbox collection
   * @param string $type The type of scheduling collection, 'inbox', 'outbox' or 'any'
@@ -1169,8 +1209,9 @@ EOQRY;
 
 
   /**
-  * Returns the URL of our resource
-  */
+   * Returns the URL of our resource
+   * @return string
+   */
   function url() {
     if ( !isset($this->dav_name) ) {
       throw Exception("What! How can dav_name not be set?");
@@ -1180,8 +1221,9 @@ EOQRY;
 
 
   /**
-  * Returns the dav_name of the resource in our internal namespace
-  */
+   * Returns the dav_name of the resource in our internal namespace
+   * @return string
+   */
   function dav_name() {
     if ( isset($this->dav_name) ) return $this->dav_name;
     return null;
@@ -1190,7 +1232,8 @@ EOQRY;
 
   /**
   * Returns the dav_name of the resource we are bound to, within our internal namespace
-  */
+   * @return string
+   */
   function bound_from() {
     if ( isset($this->bound_from) ) return $this->bound_from;
     return $this->dav_name();
@@ -1252,6 +1295,15 @@ EOQRY;
   }
 
 
+  /**
+   * Returns the name of the timezone for this collection, or the collection containing this resource
+   */
+  function timezone_name() {
+      if ( !isset($this->collection) ) $this->FetchCollection();
+      return $this->collection->timezone;
+  }
+
+
   /**
   * Returns the database row for this resource
   */
@@ -1299,7 +1351,7 @@ EOQRY;
     dbg_error_log('DAVResource', 'Request for a%scached sync-token', ($cachedOK ? ' ' : 'n un') );
     if ( $this->IsPrincipal() ) return null;
     if ( $this->collection_id() == 0 ) return null;
-    if ( !isset($this->sync_token) || !$cachedOK ) { 
+    if ( !isset($this->sync_token) || !$cachedOK ) {
       $sql = 'SELECT new_sync_token( 0, :collection_id) AS sync_token';
       $params = array( ':collection_id' => $this->collection_id());
       $qry = new AwlQuery($sql, $params );
@@ -1312,7 +1364,7 @@ EOQRY;
     dbg_error_log('DAVResource', 'Returning sync token of "%s"', $this->sync_token );
     return $this->sync_token;
   }
-  
+
   /**
   * Checks whether the target collection is publicly_readable
   */
@@ -1388,8 +1440,6 @@ EOQRY;
   * Return ACL settings
   */
   function GetACL( &$xmldoc ) {
-    global $c, $session;
-
     if ( !isset($this->principal) ) $this->FetchPrincipal();
     $default_privs = $this->principal->default_privileges;
     if ( isset($this->collection->default_privileges) ) $default_privs = $this->collection->default_privileges;
@@ -1420,8 +1470,6 @@ EOQRY;
   * Return general server-related properties, in plain form
   */
   function GetProperty( $name ) {
-    global $c, $session;
-
 //    dbg_error_log( 'DAVResource', ':GetProperty: Fetching "%s".', $name );
     $value = null;
 
@@ -1457,10 +1505,10 @@ EOQRY;
 
       case 'dav-data':
         if ( !isset($this->resource) ) $this->FetchResource();
-        trace_bug("Exists ".($this->exists?"true":"false"));
+        dbg_error_log( 'DAVResource', ':GetProperty: dav-data: fetched resource does%s exist.', ($this->exists?'':' not') );
         return $this->resource->caldav_data;
         break;
-        
+
       case 'principal':
         if ( !isset($this->principal) ) $this->FetchPrincipal();
         return clone($this->principal);
@@ -1470,7 +1518,7 @@ EOQRY;
         if ( isset($this->{$name}) ) {
           if ( ! is_object($this->{$name}) ) return $this->{$name};
           return clone($this->{$name});
-        } 
+        }
         if ( $this->_is_principal ) {
           if ( !isset($this->principal) ) $this->FetchPrincipal();
           if ( isset($this->principal->{$name}) ) return $this->principal->{$name};
@@ -1490,7 +1538,7 @@ EOQRY;
         if ( isset($this->{$name}) ) {
           if ( ! is_object($this->{$name}) ) return $this->{$name};
           return clone($this->{$name});
-        } 
+        }
         // dbg_error_log( 'DAVResource', ':GetProperty: Failed to find property "%s" on "%s".', $name, $this->dav_name );
     }
 
@@ -1502,8 +1550,8 @@ EOQRY;
   * Return an array which is an expansion of the DAV::allprop
   */
   function DAV_AllProperties() {
-    if ( isset($this->dead_properties) ) $this->FetchDeadProperties();
-    $allprop = array_merge( (isset($this->dead_properties)?$this->dead_properties:array()),
+    if ( !isset($this->dead_properties) ) $this->FetchDeadProperties();
+    $allprop = array_merge( (isset($this->dead_properties)?array_keys($this->dead_properties):array()),
       (isset($include_properties)?$include_properties:array()),
       array(
         'DAV::getcontenttype', 'DAV::resourcetype', 'DAV::getcontentlength', 'DAV::displayname', 'DAV::getlastmodified',
@@ -1524,7 +1572,7 @@ EOQRY;
     global $c, $session, $request;
 
 //    dbg_error_log( 'DAVResource', 'Processing "%s" on "%s".', $tag, $this->dav_name );
-    
+
     if ( $reply === null ) $reply = $GLOBALS['reply'];
 
     switch( $tag ) {
@@ -1648,8 +1696,9 @@ EOQRY;
 
       case 'DAV::add-member':
         if ( ! $this->_is_collection ) return false;
+        if ( $this->_is_principal ) return false;
         if ( isset($c->post_add_member) && $c->post_add_member === false ) return false;
-        $reply->DAVElement( $prop, 'add-member', $reply->href(ConstructURL(DeconstructURL($this->url())).'?add-member') );
+        $reply->DAVElement( $prop, 'add-member', $reply->href(ConstructURL(DeconstructURL($this->url())).'?add_member') );
         break;
 
       // Empty tag responses.
@@ -1681,10 +1730,34 @@ EOQRY;
         if ( isset($c->disable_caldav_proxy) && $c->disable_caldav_proxy ) return false;
         if ( !isset($proxy_type) ) $proxy_type = 'write';
         // ProxyFor is an already constructed URL
-		$this->FetchPrincipal();
+        $this->FetchPrincipal();
         $reply->CalendarserverElement($prop, 'calendar-proxy-'.$proxy_type.'-for', $reply->href( $this->principal->ProxyFor($proxy_type) ) );
         break;
 
+      case 'http://calendarserver.org/ns/:group-member-set':
+      case 'DAV::group-member-set':
+        if ( $this->_is_proxy_resource ) {
+          $this->FetchPrincipal();
+          if ( $this->proxy_type == 'read' ) {
+            $reply->DAVElement( $prop, 'group-member-set', $reply->href( $this->principal->ReadProxyGroup() ) );
+          } else {
+            $reply->DAVElement( $prop, 'group-member-set', $reply->href( $this->principal->WriteProxyGroup() ) );
+          }
+        } else {
+          return false; // leave this to DAVPrincipal
+        }
+        break;
+
+      case 'http://calendarserver.org/ns/:group-membership':
+      case 'DAV::group-membership':
+        if ( $this->_is_proxy_resource ) {
+          /* the calendar-proxy-{read,write} pseudo-principal should not be a member of any group */
+          $reply->NSElement($prop, $tag );
+        } else {
+          return false; // leave this to DAVPrincipal
+        }
+        break;
+
       case 'DAV::current-user-privilege-set':
         if ( $this->HavePrivilegeTo('DAV::read-current-user-privilege-set') ) {
           $reply->NSElement($prop, $tag, $this->BuildPrivileges() );
@@ -1759,7 +1832,7 @@ EOQRY;
         break;
 
       case 'DAV::current-user-principal':
-        $prop->NewElement('current-user-principal', $reply->href( ConstructURL(DeconstructURL($request->principal->url())) ) );
+        $prop->NewElement('current-user-principal', $reply->href( ConstructURL(DeconstructURL($session->principal->url())) ) );
         break;
 
       case 'SOME-DENIED-PROPERTY':  /** indicating the style for future expansion */
@@ -1786,7 +1859,7 @@ EOQRY;
 
       case 'urn:ietf:params:xml:ns:carddav:max-resource-size':
         if ( ! $this->_is_collection || !$this->_is_addressbook ) return false;
-        $reply->NSElement($prop, $tag, 65500 );
+        $reply->NSElement($prop, $tag, $c->carddav_max_resource_size );
         break;
 
       case 'urn:ietf:params:xml:ns:carddav:supported-address-data':
@@ -1897,8 +1970,6 @@ EOQRY;
   * @return string An XML fragment with the requested properties for this principal
   */
   function RenderAsXML( $properties, &$reply, $bound_parent_path = null ) {
-    global $session, $c, $request;
-
     dbg_error_log('DAVResource',':RenderAsXML: Resource "%s" exists(%d)', $this->dav_name, $this->Exists() );
 
     if ( !$this->Exists() ) return null;
