@@ -3,17 +3,20 @@
 * Manages LDAP repository connection
 *
 * @package   davical
-* @category Technical
-* @subpackage   ldap
+* @category  Technical
+* @subpackage authentication/drivers
 * @author    Maxime Delorme <mdelorme@tennaxia.net>,
-*   		 Andrew McMillan <andrew@mcmillan.net.nz>
+*            Andrew McMillan <andrew@mcmillan.net.nz>
 * @copyright Maxime Delorme
 * @license   http://gnu.org/copyleft/gpl.html GNU GPL v2 or later
 */
 
 require_once("auth-functions.php");
 
-class ldapDrivers
+/**
+ * Plugin to authenticate and sync with LDAP
+ */
+class ldapDriver
 {
   /**#@+
   * @access private
@@ -170,6 +173,7 @@ class ldapDrivers
           $row[$arr[$j]] = count($arr[$arr[$j]])>2?$arr[$arr[$j]]:$arr[$arr[$j]][0];
         }
         $ret[]=$row;
+        unset($row);
       }
     }
     return $ret;
@@ -257,16 +261,16 @@ function getStaticLdap() {
 
   // If the instance is not there, create one
   if(!isset($instance)) {
-    $ldapDrivers = new ldapDrivers($c->authenticate_hook['config']);
+    $ldapDriver = new ldapDriver($c->authenticate_hook['config']);
 
-    if ($ldapDrivers->valid) {
-        $instance = $ldapDrivers;
+    if ($ldapDriver->valid) {
+        $instance = $ldapDriver;
     }
   }
   else {
-      $ldapDrivers = $instance;
+      $ldapDriver = $instance;
   }
-  return $ldapDrivers;
+  return $ldapDriver;
 }
 
 
@@ -302,7 +306,7 @@ function sync_user_from_LDAP( Principal &$principal, $mapping, $ldap_values ) {
       dbg_error_log( "LDAP", "Setting usr->%s to %s from configured defaults", $field, $c->authenticate_hook['config']['default_value'][$field] );
     }
   }
-    
+
   if ( $principal->Exists() ) {
     $principal->Update($fields_to_set);
   }
@@ -313,7 +317,7 @@ function sync_user_from_LDAP( Principal &$principal, $mapping, $ldap_values ) {
   }
 }
 
-/*
+/**
 * explode the multipart mapping
 */
 function array_values_mapping($mapping){
@@ -381,7 +385,11 @@ function LDAP_check($username, $password ){
     return false;
   }
 
-  $ldap_timestamp = $valid[$mapping['modified']];
+  if ( $mapping['modified'] != "" && array_key_exists($mapping['modified'], $valid)) {
+    $ldap_timestamp = $valid[$mapping['modified']];
+  } else {
+    $ldap_timestamp = '19700101000000';
+  }
 
   /**
   * This splits the LDAP timestamp apart and assigns values to $Y $m $d $H $M and $S
@@ -390,7 +398,9 @@ function LDAP_check($username, $password ){
     $$k = substr($ldap_timestamp,$v[0],$v[1]);
 
   $ldap_timestamp = "$Y"."$m"."$d"."$H"."$M"."$S";
-  $valid[$mapping['modified']] = "$Y-$m-$d $H:$M:$S";
+  if ($mapping['modified'] != "" && array_key_exists($mapping['modified'], $valid)) {
+    $valid[$mapping['modified']] = "$Y-$m-$d $H:$M:$S";
+  }
 
   $principal = new Principal('username',$username);
   if ( $principal->Exists() ) {
@@ -406,20 +416,21 @@ function LDAP_check($username, $password ){
     dbg_error_log( "LDAP", "user %s doesn't exist in local DB, we need to create it",$username );
   }
   $principal->setUsername($username);
-  
+
   // The local cached user doesn't exist, or is older, so we create/update their details
   sync_user_from_LDAP( $principal, $mapping, $valid );
-  
+
   return $principal;
 
 }
 
+/**
+* turn a list of uniqueMember into member strings
+*/
 function fix_unique_member($list) {
   $fixed_list = array();
   foreach ( $list as $member ){
-    list( $mem, $rest ) = explode(",", $member );
-    $member = str_replace( 'uid=', '', $mem );
-    array_unshift( $fixed_list, $member );
+    array_unshift( $fixed_list, ldap_explode_dn($member,1)[0]);
   }
   return $fixed_list;
 }
@@ -440,6 +451,7 @@ function sync_LDAP_groups(){
   if ( sizeof($ldap_groups_tmp) == 0 ) return;
 
   $member_field = $mapping['members'];
+  $dnfix = isset($c->authenticate_hook['config']['group_member_dnfix']) && $c->authenticate_hook['config']['group_member_dnfix'];
 
   foreach($ldap_groups_tmp as $key => $ldap_group){
     $group_mapping = $ldap_group[$mapping['username']];
@@ -470,9 +482,14 @@ function sync_LDAP_groups(){
   $groups_to_update = array_intersect($db_groups,$ldap_groups);
 
   if ( sizeof ( $groups_to_create ) ){
-    $c->messages[] = sprintf(i18n('- creating groups : %s'),join(', ',$groups_to_create));
-    $validUserFields = get_fields('usr');
+    $validUserFields = awl_get_fields('usr');
     foreach ( $groups_to_create as $k => $group ){
+      if ( isset($c->do_not_sync_group_from_ldap) && isset($c->do_not_sync_group_from_ldap[$group]) ){
+        unset($groups_to_create[$k]);
+        $groups_nothing_done[] = $group;
+        continue;
+      }
+
       $user = (object) array();
 
       if ( isset($c->authenticate_hook['config']['default_value']) && is_array($c->authenticate_hook['config']['default_value']) ) {
@@ -512,17 +529,10 @@ function sync_LDAP_groups(){
       $qry = new AwlQuery( "UPDATE dav_principal set type_id = 3 WHERE username=:group ",array(':group'=>$group) );
       $qry->Exec('sync_LDAP',__LINE__,__FILE__);
       Principal::cacheDelete('username', $group);
-      $c->messages[] = sprintf(i18n('- adding users %s to group : %s'),join(',',$ldap_groups_info[$group][$mapping['members']]),$group);
-      foreach ( $ldap_groups_info[$group][$mapping['members']] as $member ){
-        if ( $member_field == 'uniqueMember' ) {
-          list( $mem, $rest ) = explode(",", $member );
-          $member = str_replace( 'uid=', '', $mem );
-        }
-        $qry = new AwlQuery( "INSERT INTO group_member SELECT g.principal_id AS group_id,u.principal_id AS member_id FROM dav_principal g, dav_principal u WHERE g.username=:group AND u.username=:member;",array (':group'=>$group,':member'=>$member) );
-        $qry->Exec('sync_LDAP_groups',__LINE__,__FILE__);
-        Principal::cacheDelete('username', $member);
-      }
+      // mark group for updating, so users get synced
+      $groups_to_update[] = $group;
     }
+    $c->messages[] = sprintf( i18n('- creating groups : %s'), join(', ',$groups_to_create) );
   }
 
   if ( sizeof ( $groups_to_update ) ){
@@ -530,7 +540,7 @@ function sync_LDAP_groups(){
     foreach ( $groups_to_update as $group ){
       $db_members = array_values ( $db_group_members[$group] );
       $ldap_members = array_values ( $ldap_groups_info[$group][$member_field] );
-      if ( $member_field == 'uniqueMember' ) {
+      if ( $member_field == 'uniqueMember' || $dnfix ) {
         $ldap_members = fix_unique_member( $ldap_members );
       }
       $add_users = array_diff ( $ldap_members, $db_members );
@@ -555,13 +565,21 @@ function sync_LDAP_groups(){
   }
 
   if ( sizeof ( $groups_to_deactivate ) ){
-    $c->messages[] = sprintf(i18n('- deactivate groups : %s'),join(', ',$groups_to_deactivate));
-    foreach ( $groups_to_deactivate as $group ){
-      $qry = new AwlQuery( 'UPDATE dav_principal SET user_active=FALSE WHERE username=:group AND type_id = 3',array(':group'=>$group) );
-      $qry->Exec('sync_LDAP',__LINE__,__FILE__);
-      Principal::cacheFlush('username=:group AND type_id = 3', array(':group'=>$group) );
+    foreach ( $groups_to_deactivate as $k => $group ){
+      if ( isset($c->do_not_sync_group_from_ldap) && isset($c->do_not_sync_group_from_ldap[$group]) ){
+        unset($groups_to_deactivate[$k]);
+        $groups_nothing_done[] = $group;
+      } else {
+        $qry = new AwlQuery( 'UPDATE dav_principal SET user_active=FALSE WHERE username=:group AND type_id = 3',array(':group'=>$group) );
+        $qry->Exec('sync_LDAP',__LINE__,__FILE__);
+        Principal::cacheFlush('username=:group AND type_id = 3', array(':group'=>$group) );
+      }
     }
+    if ( sizeof($groups_to_deactivate) )
+      $c->messages[] = sprintf(i18n('- deactivated groups : %s'), join(', ',$groups_to_deactivate));
   }
+  if ( sizeof($groups_nothing_done) )
+    $c->messages[] = sprintf( i18n('- nothing done on : %s'), join(', ',$groups_nothing_done) );
 
 }
 
@@ -580,14 +598,15 @@ function sync_LDAP(){
   if ( sizeof($ldap_users_tmp) == 0 ) return;
 
   foreach($ldap_users_tmp as $key => $ldap_user){
+    if(!isset($ldap_user[$mapping['username']])) continue;
     $ldap_users_info[$ldap_user[$mapping['username']]] = $ldap_user;
     unset($ldap_users_tmp[$key]);
   }
-  $qry = new AwlQuery( "SELECT username, user_no, modified as updated FROM dav_principal where type_id=1");
+  $qry = new AwlQuery( "SELECT username, user_no, modified as updated , user_active FROM dav_principal where type_id=1");
   $qry->Exec('sync_LDAP',__LINE__,__FILE__);
   while($db_user = $qry->Fetch()) {
     $db_users[] = $db_user->username;
-    $db_users_info[$db_user->username] = array('user_no' => $db_user->user_no, 'updated' => $db_user->updated);
+    $db_users_info[$db_user->username] = array('user_no' => $db_user->user_no, 'updated' => $db_user->updated, 'user_active' => $db_user->user_active);
   }
 
   // all users from ldap
@@ -601,12 +620,19 @@ function sync_LDAP(){
 
   // creation of all users;
   if ( sizeof($users_to_create) ) {
-    $c->messages[] = sprintf(i18n('- creating record for users :  %s'),join(', ',$users_to_create));
-
-    foreach( $users_to_create as $username ) {
+    foreach( $users_to_create as $k => $username ) {
+      if ( isset($c->do_not_sync_from_ldap) && isset($c->do_not_sync_from_ldap[$username]) ) {
+        unset( $users_to_create[$k] );
+        $users_nothing_done[] = $username;
+        continue;
+      }
       $principal = new Principal( 'username', $username );
       $valid = $ldap_users_info[$username];
-      $ldap_timestamp = $valid[$mapping['modified']];
+      if ( $mapping['modified'] != "" && array_key_exists($mapping['modified'], $valid)) {
+        $ldap_timestamp = $valid[$mapping['modified']];
+      } else {
+        $ldap_timestamp = '19700101000000';
+      }
 
       if ( !empty($c->authenticate_hook['config']['format_updated']) ) {
         /**
@@ -622,18 +648,25 @@ function sync_LDAP(){
       else if ( empty($ldap_timestamp) ) {
         $ldap_timestamp = date('c');
       }
-      $valid[$mapping['modified']] = $ldap_timestamp;
+      if ( $mapping['modified'] != "" && array_key_exists($mapping['modified'], $valid)) {
+        $valid[$mapping['modified']] = $ldap_timestamp;
+      }
 
       sync_user_from_LDAP( $principal, $mapping, $valid );
     }
+    $c->messages[] = sprintf( i18n('- creating record for users :  %s'), join(', ',$users_to_create) );
   }
 
   // deactivating all users
   $params = array();
   $i = 0;
   $paramstring = '';
-  foreach( $users_to_deactivate AS $v ) {
-    if ( isset($c->do_not_sync_from_ldap) && isset($c->do_not_sync_from_ldap[$v]) ) continue;
+  foreach( $users_to_deactivate as $k => $v ) {
+    if ( isset($c->do_not_sync_from_ldap) && isset($c->do_not_sync_from_ldap[$v]) ) {
+      unset($users_to_deactivate[$k]);
+      $users_nothing_done[] = $v;
+      continue;
+    }
     if ( $i > 0 ) $paramstring .= ',';
     $paramstring .= ':u'.$i.'::text';
     $params[':u'.$i++] = strtolower($v);
@@ -651,7 +684,11 @@ function sync_LDAP(){
     foreach ( $users_to_update as $key=> $username ) {
       $principal = new Principal( 'username', $username );
       $valid=$ldap_users_info[$username];
-      $ldap_timestamp = $valid[$mapping['modified']];
+      if ( $mapping['modified'] != "" && array_key_exists($mapping['modified'], $valid)) {
+        $ldap_timestamp = $valid[$mapping['modified']];
+      } else {
+        $ldap_timestamp = '19700101000000';
+      }
 
       $valid['user_no'] = $db_users_info[$username]['user_no'];
       $mapping['user_no'] = 'user_no';
@@ -666,8 +703,9 @@ function sync_LDAP(){
       $valid[$mapping['modified']] = "$Y-$m-$d $H:$M:$S";
 
       $db_timestamp = substr(strtr($db_users_info[$username]['updated'], array(':' => '',' '=>'','-'=>'')),0,14);
-      if ( $ldap_timestamp > $db_timestamp ) {
-        sync_user_from_LDAP($principal, $mapping, $valid );
+      if ( $ldap_timestamp > $db_timestamp || !$db_users_info[$username]['user_active']) {
+        $principal->user_active = true;
+        sync_user_from_LDAP($principal, $mapping, $valid);
       }
       else {
         unset($users_to_update[$key]);
@@ -676,10 +714,11 @@ function sync_LDAP(){
     }
     if ( sizeof($users_to_update) )
       $c->messages[] = sprintf(i18n('- updating user records : %s'),join(', ',$users_to_update));
-    if ( sizeof($users_nothing_done) )
-      $c->messages[] = sprintf(i18n('- nothing done on : %s'),join(', ', $users_nothing_done));
   }
+  if ( sizeof($users_nothing_done) )
+    $c->messages[] = sprintf( i18n('- nothing done on : %s'), join(', ',$users_nothing_done) );
 
+  // check for remaining admins
   $admins = 0;
   $qry = new AwlQuery( "SELECT count(*) AS admins FROM usr JOIN role_member USING ( user_no ) JOIN roles USING (role_no) WHERE usr.active=TRUE AND role_name='Admin'");
   $qry->Exec('sync_LDAP',__LINE__,__FILE__);
